@@ -61,7 +61,7 @@ typedef int adata_t;
 #define RESAMP_RATIO		64
 
 /*
- * Maximum size of the encording string (the longest possible
+ * Maximum size of the encoding string (the longest possible
  * encoding is ``s24le3msb'').
  */
 #define ENCMAX	10
@@ -74,9 +74,9 @@ typedef int adata_t;
 struct aparams {
 	unsigned int bps;		/* bytes per sample */
 	unsigned int bits;		/* actually used bits */
-	unsigned int le;		/* 1 if little endian, 0 if big endian */
+	unsigned int le;		/* 1 if little endian, else be */
 	unsigned int sig;		/* 1 if signed, 0 if unsigned */
-	unsigned int msb;		/* 1 if msb justified, 0 if lsb justified */
+	unsigned int msb;		/* 1 if msb justified, else lsb */
 };
 
 struct resamp {
@@ -93,7 +93,7 @@ struct conv {
 	int bfirst;			/* bytes to skip at startup */
 	unsigned int bps;		/* bytes per sample */
 	unsigned int shift;		/* shift to get 32bit MSB */
-	unsigned int bias;			/* bias of unsigned samples */
+	unsigned int bias;		/* bias of unsigned samples */
 	int bnext;			/* to reach the next byte */
 	int snext;			/* to reach the next sample */
 	int nch;
@@ -107,7 +107,7 @@ struct cmap {
 	int nch;
 };
 
-#define MIDI_TO_ADATA(m)	(aparams_ctltovol[m] << (ADATA_BITS - 16))
+#define MIDI_TO_ADATA(m)	(aparams_ctltovol[m])
 extern const int aparams_ctltovol[128];
 
 void aparams_init(struct aparams *);
