@@ -28,7 +28,7 @@ struct mio_hdl {
 	struct mio_ops *ops;
 	unsigned mode;			/* MIO_IN | MIO_OUT */
 	int nbio;			/* true if non-blocking io */
-	int eof;			/* true if error occured */
+	int eof;			/* true if error occurred */
 };
 
 /*
@@ -49,6 +49,5 @@ struct mio_hdl *_mio_alsa_open(const char *, unsigned, int);
 #endif
 struct mio_hdl *_mio_aucat_open(const char *, unsigned, int);
 void _mio_create(struct mio_hdl *, struct mio_ops *, unsigned, int);
-void _mio_destroy(struct mio_hdl *);
 
 #endif /* !defined(MIO_PRIV_H) */
