@@ -50,6 +50,7 @@ struct sio_aucat_hdl {
 static void sio_aucat_close(struct sio_hdl *);
 static int sio_aucat_start(struct sio_hdl *);
 static int sio_aucat_stop(struct sio_hdl *);
+static int sio_aucat_flush(struct sio_hdl *);
 static int sio_aucat_setpar(struct sio_hdl *, struct sio_par *);
 static int sio_aucat_getpar(struct sio_hdl *, struct sio_par *);
 static int sio_aucat_getcap(struct sio_hdl *, struct sio_cap *);
@@ -70,6 +71,7 @@ static struct sio_ops sio_aucat_ops = {
 	sio_aucat_read,
 	sio_aucat_start,
 	sio_aucat_stop,
+	sio_aucat_flush,
 	sio_aucat_nfds,
 	sio_aucat_pollfd,
 	sio_aucat_revents,
@@ -208,7 +210,7 @@ sio_aucat_start(struct sio_hdl *sh)
 }
 
 static int
-sio_aucat_stop(struct sio_hdl *sh)
+sio_aucat_drain(struct sio_hdl *sh, int drain)
 {
 #define ZERO_MAX 0x400
 	static unsigned char zero[ZERO_MAX];
@@ -241,6 +243,7 @@ sio_aucat_stop(struct sio_hdl *sh)
 	 */
 	AMSG_INIT(&hdl->aucat.wmsg);
 	hdl->aucat.wmsg.cmd = htonl(AMSG_STOP);
+	hdl->aucat.wmsg.u.stop.drain = drain;
 	hdl->aucat.wtodo = sizeof(struct amsg);
 	if (!_aucat_wmsg(&hdl->aucat, &hdl->sio.eof))
 		return 0;
@@ -263,6 +266,18 @@ sio_aucat_stop(struct sio_hdl *sh)
 	return 1;
 }
 
+static int
+sio_aucat_stop(struct sio_hdl *sh)
+{
+	return sio_aucat_drain(sh, 1);
+}
+
+static int
+sio_aucat_flush(struct sio_hdl *sh)
+{
+	return sio_aucat_drain(sh, 0);
+}
+
 static int
 sio_aucat_setpar(struct sio_hdl *sh, struct sio_par *par)
 {
