@@ -42,8 +42,6 @@ void dev_sub_bcopy(struct dev *, struct slot *);
 void dev_onmove(struct dev *, int);
 void dev_master(struct dev *, unsigned int);
 void dev_cycle(struct dev *);
-struct dev *dev_new(char *, struct aparams *, unsigned int, unsigned int,
-    unsigned int, unsigned int, unsigned int, unsigned int);
 void dev_adjpar(struct dev *, int, int, int);
 int dev_allocbufs(struct dev *);
 void dev_freebufs(struct dev *);
@@ -520,7 +518,8 @@ dev_mix_badd(struct dev *d, struct slot *s)
 	}
 
 	if (s->mix.resampbuf) {
-		resamp_do(&s->mix.resamp, in, s->mix.resampbuf, s->round);
+		resamp_do(&s->mix.resamp,
+		    in, s->mix.resampbuf, s->round, d->round);
 		in = s->mix.resampbuf;
 	}
 
@@ -657,7 +656,7 @@ dev_sub_bcopy(struct dev *d, struct slot *s)
 
 	if (s->sub.resampbuf) {
 		resamp_do(&s->sub.resamp,
-		    s->sub.resampbuf, resamp_out, d->round);
+		    s->sub.resampbuf, resamp_out, d->round, s->round);
 	}
 
 	if (s->sub.encbuf)
@@ -1109,7 +1108,7 @@ dev_abort(struct dev *d)
 			if (c->ops == NULL)
 				continue;
 			if (c->opt == o) {
-				c->ops->exit(s->arg);
+				c->ops->exit(c->arg);
 				c->ops = NULL;
 			}
 		}
@@ -1390,7 +1389,7 @@ dev_migrate(struct dev *odev)
 		if (s->opt == NULL || s->opt->dev != odev)
 			continue;
 		if (s->ops != NULL) {
-			s->ops->exit(s);
+			s->ops->exit(s->arg);
 			s->ops = NULL;
 		}
 	}
@@ -1778,7 +1777,7 @@ slot_new(struct opt *opt, unsigned int id, char *who,
 	s->opt = opt;
 	slot_ctlname(s, ctl_name, CTL_NAMEMAX);
 	ctl_new(CTL_SLOT_LEVEL, s, NULL,
-	    CTL_NUM, "app", ctl_name, -1, "level",
+	    CTL_NUM, "", "app", ctl_name, -1, "level",
 	    NULL, -1, 127, s->vol);
 
 found:
@@ -1930,7 +1929,7 @@ slot_attach(struct slot *s)
 	}
 
 	/*
-	 * setup converions layer
+	 * setup conversions layer
 	 */
 	slot_initconv(s);
 
@@ -2292,6 +2291,14 @@ ctlslot_visible(struct ctlslot *s, struct ctl *c)
 		return 1;
 	switch (c->scope) {
 	case CTL_HW:
+		/*
+		 * Disable hardware's server.device control as its
+		 * replaced by sndiod's one
+		 */
+		if (strcmp(c->node0.name, "server") == 0 &&
+		    strcmp(c->func, "device") == 0)
+			return 0;
+		/* FALLTHROUHG */
 	case CTL_DEV_MASTER:
 		return (s->opt->dev == c->u.any.arg0);
 	case CTL_OPT_DEV:
@@ -2335,12 +2342,14 @@ ctlslot_update(struct ctlslot *s)
 		/* nothing to do if no visibility change */
 		if (((c->refs_mask & s->self) ^ refs_mask) == 0)
 			continue;
-		/* if control becomes visble */
+		/* if control becomes visible */
 		if (refs_mask)
 			c->refs_mask |= s->self;
 		/* if control is hidden */
 		c->desc_mask |= s->self;
 	}
+	if (s->ops)
+		s->ops->sync(s->arg);
 }
 
 void
@@ -2405,6 +2414,11 @@ ctl_log(struct ctl *c)
 	default:
 		log_puts("unknown");
 	}
+	if (c->display[0] != 0) {
+		log_puts(" (");
+		log_puts(c->display);
+		log_puts(")");
+	}
 }
 
 int
@@ -2450,8 +2464,8 @@ ctl_setval(struct ctl *c, int val)
 		c->curval = val;
 		return 1;
 	case CTL_OPT_DEV:
-		c->u.opt_dev.opt->alt_first = c->u.opt_dev.dev;
-		opt_setdev(c->u.opt_dev.opt, c->u.opt_dev.dev);
+		if (opt_setdev(c->u.opt_dev.opt, c->u.opt_dev.dev))
+			c->u.opt_dev.opt->alt_first = c->u.opt_dev.dev;
 		return 1;
 	default:
 		if (log_level >= 2) {
@@ -2467,7 +2481,7 @@ ctl_setval(struct ctl *c, int val)
  */
 struct ctl *
 ctl_new(int scope, void *arg0, void *arg1,
-    int type, char *gstr,
+    int type, char *display, char *gstr,
     char *str0, int unit0, char *func,
     char *str1, int unit1, int maxval, int val)
 {
@@ -2491,6 +2505,7 @@ ctl_new(int scope, void *arg0, void *arg1,
 	c->type = type;
 	strlcpy(c->func, func, CTL_NAMEMAX);
 	strlcpy(c->group, gstr, CTL_NAMEMAX);
+	strlcpy(c->display, display, CTL_DISPLAYMAX);
 	strlcpy(c->node0.name, str0, CTL_NAMEMAX);
 	c->node0.unit = unit0;
 	if (c->type == CTL_VEC || c->type == CTL_LIST || c->type == CTL_SEL) {
@@ -2549,11 +2564,12 @@ ctl_update(struct ctl *c)
 		/* nothing to do if no visibility change */
 		if (((c->refs_mask & s->self) ^ refs_mask) == 0)
 			continue;
-		/* if control becomes visble */
+		/* if control becomes visible */
 		if (refs_mask)
 			c->refs_mask |= s->self;
 		/* if control is hidden */
 		c->desc_mask |= s->self;
+		s->ops->sync(s->arg);
 	}
 }
 
@@ -2602,16 +2618,18 @@ ctl_onval(int scope, void *arg0, void *arg1, int val)
 	return 1;
 }
 
-void
+int
 ctl_del(int scope, void *arg0, void *arg1)
 {
 	struct ctl *c, **pc;
+	int found;
 
+	found = 0;
 	pc = &ctl_list;
 	for (;;) {
 		c = *pc;
 		if (c == NULL)
-			return;
+			return found;
 		if (ctl_match(c, scope, arg0, arg1)) {
 #ifdef DEBUG
 			if (log_level >= 2) {
@@ -2619,6 +2637,7 @@ ctl_del(int scope, void *arg0, void *arg1)
 				log_puts(": removed\n");
 			}
 #endif
+			found++;
 			c->refs_mask &= ~CTL_DEVMASK;
 			if (c->refs_mask == 0) {
 				*pc = c->next;
@@ -2632,11 +2651,32 @@ ctl_del(int scope, void *arg0, void *arg1)
 	}
 }
 
+char *
+dev_getdisplay(struct dev *d)
+{
+	struct ctl *c;
+	char *display;
+
+	display = "";
+	for (c = ctl_list; c != NULL; c = c->next) {
+		if (c->scope == CTL_HW &&
+		    c->u.hw.dev == d &&
+		    c->type == CTL_SEL &&
+		    strcmp(c->group, d->name) == 0 &&
+		    strcmp(c->node0.name, "server") == 0 &&
+		    strcmp(c->func, "device") == 0 &&
+		    c->curval == 1)
+			display = c->display;
+	}
+	return display;
+}
+
 void
 dev_ctlsync(struct dev *d)
 {
 	struct ctl *c;
 	struct ctlslot *s;
+	const char *display;
 	int found, i;
 
 	found = 0;
@@ -2664,10 +2704,23 @@ dev_ctlsync(struct dev *d)
 		}
 		d->master_enabled = 1;
 		ctl_new(CTL_DEV_MASTER, d, NULL,
-		    CTL_NUM, d->name, "output", -1, "level",
+		    CTL_NUM, "", d->name, "output", -1, "level",
 		    NULL, -1, 127, d->master);
 	}
 
+	/*
+	 * if the hardware's server.device changed, update the display name
+	 */
+	display = dev_getdisplay(d);
+	for (c = ctl_list; c != NULL; c = c->next) {
+		if (c->scope != CTL_OPT_DEV ||
+		    c->u.opt_dev.dev != d ||
+		    strcmp(c->display, display) == 0)
+			continue;
+		strlcpy(c->display, display, CTL_DISPLAYMAX);
+		c->desc_mask = ~0;
+	}
+
 	for (s = ctlslot_array, i = 0; i < DEV_NCTLSLOT; i++, s++) {
 		if (s->ops == NULL)
 			continue;
