@@ -155,9 +155,11 @@ struct ctl {
 	} u;
 
 	unsigned int addr;		/* slot side control address */
-#define CTL_NAMEMAX	16		/* max name lenght */
+#define CTL_NAMEMAX	12		/* max name length */
+#define CTL_DISPLAYMAX	24		/* max name length */
 	char func[CTL_NAMEMAX];		/* parameter function name */
 	char group[CTL_NAMEMAX];	/* group aka namespace */
+	char display[CTL_DISPLAYMAX];	/* free-format hint */
 	struct ctl_node {
 		char name[CTL_NAMEMAX];	/* stream name */
 		int unit;
@@ -301,7 +303,6 @@ int  dev_init(struct dev *);
 void dev_done(struct dev *);
 int dev_ref(struct dev *);
 void dev_unref(struct dev *);
-int  dev_getpos(struct dev *);
 unsigned int dev_roundof(struct dev *, unsigned int);
 int dev_iscompat(struct dev *, struct dev *);
 
@@ -351,8 +352,8 @@ void slot_detach(struct slot *);
  */
 
 struct ctl *ctl_new(int, void *, void *,
-    int, char *, char *, int, char *, char *, int, int, int);
-void ctl_del(int, void *, void *);
+    int, char *, char *, char *, int, char *, char *, int, int, int);
+int ctl_del(int, void *, void *);
 void ctl_log(struct ctl *);
 int ctl_setval(struct ctl *c, int val);
 int ctl_match(struct ctl *, int, void *, void *);
@@ -366,7 +367,7 @@ int ctlslot_visible(struct ctlslot *, struct ctl *);
 struct ctl *ctlslot_lookup(struct ctlslot *, int);
 void ctlslot_update(struct ctlslot *);
 
-void dev_label(struct dev *, int);
+char *dev_getdisplay(struct dev *);
 void dev_ctlsync(struct dev *);
 
 #endif /* !defined(DEV_H) */
