@@ -47,7 +47,7 @@ struct opt *opt_byname(char *);
 struct opt *opt_bynum(int);
 void opt_init(struct opt *);
 void opt_done(struct opt *);
-void opt_setdev(struct opt *, struct dev *);
+int opt_setdev(struct opt *, struct dev *);
 struct dev *opt_ref(struct opt *);
 void opt_unref(struct opt *);
 
